Option Strict Off
Option Explicit On
Imports System.Runtime.InteropServices  'For use of Marshal attributes for memory usage

Module Module1
    'Based upon code from takisfmf.gr
    'Modified for own purposes by pa2ohh
    'Modified for Marshal attributes for Memory usage in .NET

    Public Const WHDR_DONE As Integer = &H1     'Done bit

    'Declare WAVEHDR structure used with winmm.dll
	Structure WAVEHDR
		Dim lpData As Integer
		Dim dwBufferLength As Integer
		Dim dwBytesRecorded As Integer
		Dim dwUser As Integer
		Dim dwFlags As Integer
		Dim dwLoops As Integer
		Dim lpNext As Integer
		Dim Reserved As Integer
	End Structure
	
    'Declare WAVEFORMAT structure used with winmm.dll
    Structure WAVEFORMAT
        Dim wFormatTag As Short
        Dim nChannels As Short
        Dim nSamplesPerSec As Integer
        Dim nAvgBytesPerSec As Integer
        Dim nBlockAlign As Short
        Dim wBitsPerSample As Short
        Dim cbSize As Short
    End Structure
	
    'Declare functions used with winmm.dll
    Declare Function waveInOpen Lib "winmm.dll" (ByRef lphWaveIn As Integer, ByVal uDeviceID As Integer, ByRef lpFormat As WAVEFORMAT, ByVal dwCallback As Integer, ByVal dwInstance As Integer, ByVal dwFlags As Integer) As Integer
    Declare Function waveInStart Lib "winmm.dll" (ByVal hWaveIn As Integer) As Integer
    Declare Function waveInStop Lib "winmm.dll" (ByVal hWaveIn As Integer) As Integer
    Declare Function waveInAddBuffer Lib "winmm.dll" (ByVal hWaveIn As Integer, ByRef lpWaveInHdr As WAVEHDR, ByVal uSize As Integer) As Integer
    Declare Function waveInReset Lib "winmm.dll" (ByVal hWaveIn As Integer) As Integer
    Declare Function waveInPrepareHeader Lib "winmm.dll" (ByVal hWaveIn As Integer, ByRef lpWaveInHdr As WAVEHDR, ByVal uSize As Integer) As Integer
    Declare Function waveInGetErrorText Lib "winmm.dll" Alias "waveInGetErrorTextA" (ByVal errorIN As Integer, ByVal lpText As String, ByVal uSize As Integer) As Integer
    Declare Function waveInClose Lib "winmm.dll" (ByVal hWaveIn As Integer) As Integer
    Declare Function waveInUnprepareHeader Lib "winmm.dll" (ByVal hWaveIn As Integer, ByRef lpWaveInHdr As WAVEHDR, ByVal uSize As Integer) As Integer

    'Declare general variables
    Public fRecording As Boolean            'Is True if the recording is running
    Public formatW As WAVEFORMAT            'The format of the Audio stream
    Public inHdr As WAVEHDR                 'The number of samples to be recorded, buffer length and status etc.
    Public Const DEVICEID As Short = 0      'Device ID = 0 for default Windows recording device, can be changed for other input devices
    Public BufferMemory As IntPtr           'Pointer to reserved memory for audio buffer
    Public errorVal As Integer              'Error Values
    Public hWaveIn As Integer               'Device handle
    Public intStatus As Integer             'Status variable, becomes 1 if recording has been finished
    Public BUFFER_SIZE As Integer           'Buffer size that has to be filled with audio samples
    Public SkipAudioSamples As Integer      'These samples are the skipped start of the audio buffer

    Sub StartInput()
        'Open audio stream and do the recording of the audio FFT samples (bins)
        Dim msg As String

        If fRecording Then      'Not necessary but leave it here for future modifications
            Exit Sub
        End If

        formatW.wFormatTag = 1
        formatW.nChannels = 1
        formatW.wBitsPerSample = 16
        formatW.nSamplesPerSec = Form1.intSampleRate    'Get the sample rate from the main form variable intSampleRate
        'formatW.nSamplesPerSec = Val(Form1.cboSRate.Text) 'Get the sample rate from the main form from ComboBox cboSRate.Text
        formatW.nBlockAlign = formatW.nChannels * formatW.wBitsPerSample / 8
        formatW.nAvgBytesPerSec = formatW.nSamplesPerSec * formatW.nBlockAlign
        formatW.cbSize = 0

        BufferMemory = Marshal.AllocHGlobal(BUFFER_SIZE)
        inHdr.lpData = BufferMemory
        inHdr.dwBufferLength = BUFFER_SIZE
        inHdr.dwBytesRecorded = 0
        inHdr.dwFlags = 0
        inHdr.dwLoops = 0

        'Open the device. DEVICEID=0 is the Windows default input that can be selected in the audio mixer settings
        errorVal = waveInOpen(hWaveIn, DEVICEID, formatW, 0, 0, 0)
        If errorVal <> 0 Then
            msg = Space(100)                            'Make string with length of 100 characters for error message
            waveInGetErrorText(errorVal, msg, Len(msg))
            MsgBox(msg)
            Exit Sub
        End If

        errorVal = waveInPrepareHeader(hWaveIn, inHdr, Marshal.SizeOf(inHdr))

        If (errorVal <> 0) Then
            msg = Space(100)                            'Make string with length of 100 characters for error message
            waveInGetErrorText(errorVal, msg, Len(msg))
            MsgBox(msg)
            Exit Sub
        End If
        errorVal = waveInStart(hWaveIn)

        'Do the reading of the audio samples into the Memory buffer
        errorVal = waveInAddBuffer(hWaveIn, inHdr, Marshal.SizeOf(inHdr))
        If (errorVal <> 0) Then
            msg = Space(100)                            'Make string with length of 100 characters for error message
            waveInGetErrorText(errorVal, msg, Len(msg))
            MsgBox(msg)
            Exit Sub
        End If

        fRecording = True                               'The recording is running now

        intStatus = 0                                   'The status of the reading, intStatus = 1 if finished
        Do While intStatus = 0
            intStatus = (inHdr.dwFlags And WHDR_DONE)   'Wait till reading audio finished (logical And with bit 1 of inHdr.dwFlags)
        Loop

        If inHdr.dwBytesRecorded <> BUFFER_SIZE Then    'Useless, but just a check to understand how the recording works
            MsgBox("Error: Wrong number of audio samples!")
        End If

    End Sub

    Sub StopInput()
        ' Close the audio stream and free the memory, so read the samples before the StopInput() call!!!
        waveInReset(hWaveIn)
        waveInStop(hWaveIn)
        waveInUnprepareHeader(hWaveIn, inHdr, Len(inHdr))
        waveInClose(hWaveIn)
        Marshal.FreeHGlobal(BufferMemory)           'Free the memory

        fRecording = False
    End Sub

    Sub GetMono16Sample(ByVal sample As Integer, ByRef leftVol As Double)
        'Get an audio sample and return it as double in the leftVol variable ByRef
        Dim sample16 As Short

        sample16 = Marshal.ReadInt16(BufferMemory, formatW.nBlockAlign * sample)
        leftVol = CDbl(sample16 / 32768)        'The value of the audio sample converted to -1 / +1 interval and Double

    End Sub
	
	'The following code may be from the original program author
	'but it appears to be a relatively generic adaptation.
	'This routine imposes a speed bottleneck when the number
	'of FFT points is > 512.
	
	'THE FAST FOURIER TRANSFORM
	'Upon entry, N% contains the number of points in the DFT,
	'REX[ ] and IMX[ ] contain the real and imaginary parts of the input.
	'Upon return, REX[ ] and IMX[ ] contain the DFT output.
	'All signals run from 0 to N-1.
	
    Sub FFT(ByRef N As Integer, ByRef REX() As Double, ByRef IMX() As Double)
        Const Pi As Double = Math.PI
        Dim NM1 As Integer
        Dim ND2 As Integer
        Dim M As Integer
        Dim j As Integer
        Dim K As Integer
        Dim L As Integer
        Dim LE As Integer
        Dim LE2 As Integer
        Dim JM1 As Integer
        Dim i As Integer
        Dim IP As Integer
        Dim TR As Double
        Dim TI As Double
        Dim UR As Double
        Dim UI As Double
        Dim SR As Double
        Dim SI As Double

        NM1 = N - 1
        ND2 = N / 2
        M = CInt(System.Math.Log(N) / System.Math.Log(2))
        j = ND2

        For i = 1 To N - 2                              'Bit reversal sorting
            If i < j Then
                TR = REX(j)
                TI = IMX(j)
                REX(j) = REX(i)
                IMX(j) = IMX(i)
                REX(i) = TR
                IMX(i) = TI

            End If

            K = ND2

            Do While K <= j
                j = j - K
                K = K / 2
            Loop

            j = j + K
        Next

        For L = 1 To M                                  'Loop for each stage
            LE = CInt(2 ^ L)
            LE2 = LE / 2
            UR = 1
            UI = 0
            SR = System.Math.Cos(Pi / LE2)              'Calculate sine & cosine values
            SI = -System.Math.Sin(Pi / LE2)

            For j = 1 To LE2                            'Loop for each sub DFT
                JM1 = j - 1
                For i = JM1 To NM1 Step LE              'Loop for each butterfly
                    IP = i + LE2
                    TR = REX(IP) * UR - IMX(IP) * UI    'Butterfly calculation
                    TI = REX(IP) * UI + IMX(IP) * UR
                    REX(IP) = REX(i) - TR
                    IMX(IP) = IMX(i) - TI
                    REX(i) = REX(i) + TR
                    IMX(i) = IMX(i) + TI
                Next
                TR = UR
                UR = TR * SR - UI * SI
                UI = TR * SI + UI * SR
            Next
        Next

    End Sub
End Module